<?php

namespace The99Bitcoins\BtcMiningCalculator;

class Plugin {
	protected $config
		= array(
			'url'       => '99bitcoins-btc-mining-calculator',
			'plugin'    => '99bitcoins-btc-mining-calculator/',
			'templates' => '../templates/',
		);

	public function __construct( $config = array() ) {
		$this->config = $config + $this->config;
		add_action( 'admin_menu', array( $this, 'backendActionMenu' ) );
		add_action( 'init', array( $this, 'init' ) );
	}

	public function init() {
		add_shortcode( 'btc-mining-calculator', [ $this, 'frontendShortcodeMiningCalculator' ] );
		add_shortcode( 'btc-price', [ $this, 'frontendShortcodePrice' ] );

		$locale = get_locale();
		load_textdomain( '99btc-bmc', WP_LANG_DIR . '/plugins/99btc-bmc-' . $locale . '.mo' );
		load_plugin_textdomain( '99btc-bmc', false, $this->config['plugin'] . 'languages/' );
	}

	public static function cron() {
		$curl = new \WP_Http_Curl();
		$data = $curl->request( 'https://99bitcoins.com/btc.json' );
		if ( $data['response']['code'] == 200 ) {
			$data = json_decode( $data['body'], true );

			$options                              = get_option( __CLASS__ );
			$options['current_difficulty_factor'] = $data['difficulty'];
			$options['btc_exchange_rate']         = $data['price_usd'];
			update_option( __CLASS__, $options, 'no' );
		}
	}

	public function backendActionMenu() {
		add_menu_page( translate( 'BTC mining calculator', '99btc-bmc' ),
			translate( 'BTC mining calculator', '99btc-bmc' ), 'manage_options', $this->config['url'],
			array( $this, 'backendPageConfig' ), '', null );
		add_submenu_page( $this->config['url'], translate( 'Config', '99btc-bmc' ), translate( 'Config', '99btc-bmc' ),
			'manage_options', $this->config['url'], array( $this, 'backendPageConfig' ) );
		add_submenu_page( $this->config['url'], translate( 'Tooltips', '99btc-bmc' ),
			translate( 'Tooltips', '99btc-bmc' ), 'manage_options', $this->config['url'] . '/tooltips',
			array( $this, 'backendPageTooltips' ) );
		add_submenu_page( $this->config['url'], translate( 'How to use', '99btc-bmc' ),
			translate( 'How to use', '99btc-bmc' ), 'manage_options', $this->config['url'] . '/how-to',
			array( $this, 'backendPageHowToUse' ) );
	}

	public function backendPageConfig() {
		$variables = get_option( __CLASS__ );
		if ( $_SERVER['REQUEST_METHOD'] == 'POST' ) {
			$options = get_option( __CLASS__ );
			if ( isset( $_POST['current_difficulty_factor'] ) ) {
				$options['current_difficulty_factor'] = $_POST['current_difficulty_factor'];
			}
			if ( isset( $_POST['btc_reward'] ) ) {
				$options['btc_reward'] = $_POST['btc_reward'];
			}
			if ( isset( $_POST['btc_exchange_rate'] ) ) {
				$options['btc_exchange_rate'] = $_POST['btc_exchange_rate'];
			}
			if ( isset( $_POST['btc_hide_copyright'] ) ) {
				$options['btc_hide_copyright'] = $_POST['btc_hide_copyright'];
			}
			update_option( __CLASS__, $options, 'no' );
			$variables                     = array_merge( $variables, $options );
			$variables['notice_css_class'] = 'updated notice';
			$variables['notice_message']   = translate('Changes have been saved', '99btc-bmc');
		}

		$this->render( 'config', $variables );
	}

	public function backendPageTooltips() {
		$variables           = get_option( __CLASS__ );
		$variables['fields'] = array(
			array(
				'name'        => 'tooltip_difficulty_factor',
				'title'       => 'Difficulty factor',
				'placeholder' => 'Tooltip for difficulty factor',
			),
			array(
				'name'        => 'tooltip_hash_rate',
				'title'       => 'Hash rate',
				'placeholder' => 'Tooltip for hash rate',
			),
			array(
				'name'        => 'tooltip_btc_reward',
				'title'       => 'BTC/Block reward',
				'placeholder' => 'Tooltip for BTC/Block reward',
			),
			array(
				'name'        => 'tooltip_btc_exchange_rate',
				'title'       => 'USD/BTC exchange rate',
				'placeholder' => 'Tooltip for USD/BTC exchange rate',
			),
			array(
				'name'        => 'tooltip_pool_fee',
				'title'       => 'Pool Fees %',
				'placeholder' => 'Tooltip for pool fees',
			),
			array(
				'name'        => 'tooltip_power_watts',
				'title'       => 'Power (Watts)',
				'placeholder' => 'Tooltip for power',
			),
			array(
				'name'        => 'tooltip_power_cost',
				'title'       => 'Power Cost (USD/kWh)',
				'placeholder' => 'Tooltip for power cost',
			),
			array(
				'name'        => 'tooltip_hardware_costs',
				'title'       => 'Hardware Costs (USD)',
				'placeholder' => 'Tooltip for hardware costs',
			),
		);

		if ( $_SERVER['REQUEST_METHOD'] == 'POST' ) {
			$options = get_option( __CLASS__ );
			foreach ( $variables['fields'] as $field ) {
				if ( isset( $_POST[ $field['name'] ] ) ) {
					$options[ $field['name'] ] = stripslashes( $_POST[ $field['name'] ] );
				}
			}
			update_option( __CLASS__, $options, 'no' );
			$variables                     = array_merge( $variables, $options );
			$variables['notice_css_class'] = 'updated notice';
			$variables['notice_message']   = 'Changes have been saved';
		}
		$this->render( 'tooltips', $variables );
	}

	public function backendPageHowToUse() {
		$this->render( 'how-to-use' );
	}

	public function frontendShortcodeMiningCalculator() {
		$variables = get_option( __CLASS__ );

		$variables['hash_grades']     = [
			'KH/s' => '1000',
			'MH/s' => '1000000',
			'GH/s' => '1000000000',
			'TH/s' => '1000000000000',
			'PH/s' => '1000000000000000',
		];
		$variables['days']            = [
			'1 day'   => '1',
			'1 week'  => '7',
			'2 weeks' => '14',
			'3 weeks' => '21',
			'1 month' => '30',
		];
		$variables['hash_rate_grade'] = 'TH/s';
		$variables['result']          = [ ];

		if ( $_SERVER['REQUEST_METHOD'] == 'GET' && isset( $_GET['action'] )
		     && $_GET['action'] == '99btc-bmc-calucalate'
		) {
			if ( ! empty( $_GET['current_difficulty_factor'] ) ) {
				$variables['current_difficulty_factor'] = $_GET['current_difficulty_factor'];
			}
			if ( ! empty( $_GET['hash_rate'] ) ) {
				$variables['hash_rate'] = $_GET['hash_rate'];
			}
			if ( ! empty( $_GET['hash_rate_grade'] ) ) {
				$variables['hash_rate_grade'] = $_GET['hash_rate_grade'];
			}
			if ( ! empty( $_GET['btc_reward'] ) ) {
				$variables['btc_reward'] = $_GET['btc_reward'];
			}
			if ( ! empty( $_GET['btc_exchange_rate'] ) ) {
				$variables['btc_exchange_rate'] = $_GET['btc_exchange_rate'];
			}
			if ( ! empty( $_GET['pool_fee'] ) && is_numeric( $_GET['pool_fee'] ) ) {
				$variables['pool_fee'] = $_GET['pool_fee'];
			}
			if ( ! empty( $_GET['power_watts'] ) && is_numeric( $_GET['power_watts'] ) ) {
				$variables['power_watts'] = $_GET['power_watts'];
			}
			if ( ! empty( $_GET['power_cost'] ) && is_numeric( $_GET['power_cost'] ) ) {
				$variables['power_cost'] = $_GET['power_cost'];
			}
			if ( ! empty( $_GET['hardware_costs'] ) && is_numeric( $_GET['hardware_costs'] ) ) {
				$variables['hardware_costs'] = $_GET['hardware_costs'];
			}
			if ( is_numeric( $variables['current_difficulty_factor'] ) ) {
				foreach ( $variables['days'] as $k => $days ) {
					$hashRate = bcmul( $variables['hash_rate'],
						$variables['hash_grades'][ $variables['hash_rate_grade'] ] );

					$top = $days;
					$top = bcmul( $top, $variables['btc_reward'], 10 );
					$top = bcmul( $top, $hashRate );
					$top = bcmul( $top, '86400', 10 );

					$bottom = bcmul( '4294967296', $variables['current_difficulty_factor'], 10 );

					$variables['result'][ $k ]['btc']        = bcdiv( $top, $bottom, 8 );
					$variables['result'][ $k ]['fee']        = bcmul( $variables['result'][ $k ]['btc'],
						$variables['pool_fee'] / 100, 8 );
					$variables['result'][ $k ]['profit_btc'] = bcsub( $variables['result'][ $k ]['btc'],
						$variables['result'][ $k ]['fee'], 8 );
					$variables['result'][ $k ]['profit_usd'] = bcmul( $variables['result'][ $k ]['profit_btc'],
						$variables['btc_exchange_rate'], 2 );
					if ( $variables['hardware_costs'] ) {
						$variables['result'][ $k ]['profit_usd'] = bcsub( $variables['result'][ $k ]['profit_usd'],
							$variables['hardware_costs'], 2 );
					}
					if ( $variables['power_watts'] && $variables['power_cost'] ) {
						$variables['result'][ $k ]['power_kwh']   = bcdiv( $days, 1000, 10 );
						$variables['result'][ $k ]['power_kwh']   = bcmul( '24',
							$variables['result'][ $k ]['power_kwh'], 10 );
						$variables['result'][ $k ]['power_kwh']   = bcmul( $variables['power_watts'],
							$variables['result'][ $k ]['power_kwh'], 10 );
						$variables['result'][ $k ]['power_price'] = bcmul( $variables['result'][ $k ]['power_kwh'],
							$variables['power_cost'], 2 );
						$variables['result'][ $k ]['profit_usd']  = bcsub( $variables['result'][ $k ]['profit_usd'],
							$variables['result'][ $k ]['power_price'], 2 );
					}
				}
			}
		}

		ob_start();
		$this->render( 'mining-calculator', $variables );
		$content = ob_get_contents();
		ob_end_clean();

		return trim( $content );
	}

	public function frontendShortcodePrice( $args, $body, $code ) {
		$options = get_option( __CLASS__ );

		return number_format( $options['btc_exchange_rate'], 2, '.', ',' );
	}

	protected function render( $_template, $_vairables = array(), $_layout = '' ) {
		extract( $_vairables );
		ob_start();
		include $this->config['templates'] . $_template . '.php';
		$_content = ob_get_contents();
		ob_end_clean();
		if ( $_layout ) {
			include $this->config['tempaltes'] . 'layout' . DIRECTORY_SEPARATOR . $_layout . '.php';
		} else {
			echo $_content;
		}
	}
}
